package email

import (
	"fmt"
	"gn/gn_email_listen/config"
	"gn/gn_email_listen/global"
	"gn/gn_email_listen/models/report"
	"gn/gn_email_listen/services"
	"gn/gn_email_listen/utils"
	"gn/gn_email_listen/utils/mail"
	"io/fs"
	"log"
	"os"
	"regexp"
	"sync"
	"time"
)

// 同步用户锁，防止重复同步，不管是全量还是增量，都是同一时间只能一个同步
var lockListenEmail sync.Mutex

func ListenMail() {
	var err error
	defer func() {
		if err != nil {
			global.FILE_LOG.Errorf("监听邮件失败：%s", err.Error())
		}
	}()
	//Fix()
	//return
	lockListenEmail.Lock()
	// 目录创建
	_ = ensureDirExists(fmt.Sprintf("%s%s", global.CONFIG.Serve.StaticDir, `file`))

	mailMessageChan := make(chan mail.MailMessage, 5) // 创建一个通道，用于接收邮件消息
	mailMessageDoneChan := make(chan bool, 1)         // 创建一个通道，用于接收邮件消息

	// 邮件监听后的处理函数
	go afterByListen(mailMessageChan, mailMessageDoneChan)

	fmt.Println("开始监听邮件")

	emailMessageUID := global.CONFIG.Email.StartReadIndex
	log.Println("配置起始监听Email id", emailMessageUID)
	if emailMessageUID <= 0 {
		// 获取最大的邮件id
		emailMessageUID, err = report.GetMaxOutsideReportByEmailMessageId()
		// 已经存在了，那么就返回
		if err != nil {
			global.FILE_LOG.Errorf("获取已入库的最大邮件id失败：%s", err.Error())
		}
	}
	log.Println("运行时，起始的监听Email id", emailMessageUID)

	mailAddress := fmt.Sprintf("%s:%d", global.CONFIG.Email.Host, global.CONFIG.Email.Port)
	mail.ListenMail(mailAddress, global.CONFIG.Email.Folder, global.CONFIG.Email.UserName, global.CONFIG.Email.Password, global.CONFIG.Email.ReadBatchSize, emailMessageUID, mailMessageChan, mailMessageDoneChan)

}

func afterByListen(mailMessageChan chan mail.MailMessage, mailMessageDoneChan chan bool) {
	defer func() {
		log.Println("监听读取结束")
		lockListenEmail.Unlock()
	}()
	for {
		select {
		case emailMessage := <-mailMessageChan:
			// fmt.Println("读取成功")
			// fmt.Println(emailMessage.Title)
			handleMailMessage(emailMessage)
		case <-time.After(60 * time.Minute):
			// fmt.Println("监听超时了")
			return
		case <-mailMessageDoneChan:
			// fmt.Println("读取完成一轮了")
			for len(mailMessageChan) > 0 {
				emailMessage := <-mailMessageChan
				handleMailMessage(emailMessage)
			}
			// fmt.Println("结束了")
			return
		}
	}
}

func handleMailMessage(emailMessage mail.MailMessage) (err error) {
	defer func() {
		if err != nil {
			global.FILE_LOG.Errorf("邮件处理失败，邮件标题：%s，错误原因：%v", emailMessage.Title, err)
		}

		for _, v := range emailMessage.Resources {
			os.Remove(v)
		}
	}()

	emailMessageUID := int(emailMessage.Uid)
	outReport, err := report.GetListenReportLogByEmailMessageId(emailMessageUID)
	// 已经存在了，那么就返回
	if err == nil {
		global.FILE_LOG.Debugf("已存在，就不处理了,报告标题：%s；；邮件下标：%d", outReport.Title, emailMessage.Uid)
		return
	}
	fmt.Println("开始处理邮件，标题：", emailMessage.Title, "；邮件下标：", emailMessage.Uid)
	// sql报错，那么就返回
	if err != nil && !utils.IsErrNoRow(err) {
		return
	}
	// 已经存在了，那么就返回
	if outReport.Id > 0 {
		return
	}

	title := emailMessage.Title
	rule, err := MatchTitleRule(emailMessage.Title, emailMessage.From)
	if err != nil {
		return
	}
	if rule == nil {
		return
	}
	// 处理附件

	params := make(map[string]string)
	files := make(map[string][]byte)
	for k, v := range emailMessage.Attachment {
		ok, err := regexp.MatchString(rule.FilterFileType, k)
		if err != nil {
			global.FILE_LOG.Error("FilterFileType正则匹配失败：%v", err)
			continue
		}
		if !ok {
			continue
		}
		files[k] = v
	}
	params["title"] = title
	params["source"] = rule.Source
	params["classify_path"] = rule.ClassifyPath
	params["push_time"] = emailMessage.Date.Format(utils.FormatDateTime)
	resp, err := services.SendFilesHttp(global.CONFIG.Serve.BridgeUrl+"knowledge/report/save", files, params)
	if err != nil {
		global.FILE_LOG.Error("发送文件失败：%v", err)
		return
	}
	if resp.Code != 200 {
		global.FILE_LOG.Info("请求bridge服务失败：%s", resp.Msg)
		return
	}
	reportInfo := &report.ListenReportLog{
		Title:           title,
		Author:          emailMessage.From,
		Email:           emailMessage.FromAddress,
		EmailMessageUID: emailMessageUID,
		CreateTime:      time.Now(),
	}

	err = report.CreateListenReportLog(reportInfo)

	return
}

func MatchTitleRule(title, from string) (*config.EmailRule, error) {
	var err error
	for _, v := range global.CONFIG.EmailRule {
		ok, er := regexp.Match(v.Pattern, []byte(title))
		if er != nil {
			global.FILE_LOG.Error("正则匹配失败：%v", er)
			err = er
			continue
		}
		if ok && (v.From == from || v.From == "" || v.From == "*") {
			return &v, nil
		}
	}
	return nil, err
}

func ensureDirExists(dirPath string) error {
	info, err := os.Stat(dirPath)
	if err == nil {
		if info.IsDir() {
			return nil // 目录已存在
		}
		return fmt.Errorf("path '%s' exists but is not a directory", dirPath)
	}
	if os.IsNotExist(err) {
		return os.MkdirAll(dirPath, fs.ModePerm)
	}
	return err
}
